<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/hashed_array.php';
require_once 'lib/tv/abstract_tv.php';
require_once 'lib/tv/default_epg_item.php';
require_once 'lib/tv/epg_iterator.php';

require_once 'ipnet_channel.php';

///////////////////////////////////////////////////////////////////////////

class IpnetTv extends AbstractTv
{
    private $session;

    ///////////////////////////////////////////////////////////////////////

    public function __construct($session)
    {
        $this->session = $session;

        parent::__construct(
            AbstractTv::MODE_CHANNELS_N_TO_M,
            true,
            false);
    }

    public function get_fav_icon_url()
    {
        return $this->session->get_icon('favorites.png');
    }

    ///////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////

    protected function load_channels(&$plugin_cookies)
    {
        $this->channels = new HashedArray();
        $this->groups = new HashedArray();

        $this->groups->put(
            new FavoritesGroup(
                $this,
                '__favorites',
                'Избранное',
                $this->session->get_icon('favorites.png')));

        $this->groups->put(
            new AllChannelsGroup(
                $this,
                'Все каналы',
                $this->session->get_icon('all.png')));

        $num_groups = 0;
        $num_channels = 0;
        $num_protected = 0;
        $num_have_archive = 0;
        foreach ($this->session->get_channel_list()->groups->item as $g)
        {
            $group_id = strval($g->id);
            $group_name = strval($g->name);

            $ipnet_group = new DefaultGroup(
                $group_id,
                $group_name,
                $this->session->get_group_icon($group_id));

            foreach ($g->channels->item as $c)
            {
                $channel_id = strval($c->id);
                $channel_name = strval($c->name);

                $have_archive = isset($c->have_archive) ?
                    (intval($c->have_archive) > 0) : false;
                $is_protected = isset($c->protected) ?
                    (intval($c->protected) == 1) : false;
                
                $ipnet_channel = new IpnetChannel(
                    $channel_id,
                    $channel_name,
                    $this->session->get_channel_icon($channel_id),
                    $have_archive, $is_protected,
                    3000, intval($c->have_archive));
                $this->channels->put($ipnet_channel);

                $ipnet_channel->add_group($ipnet_group);
                $ipnet_group->add_channel($ipnet_channel);

                $num_channels++;
                if ($is_protected)
                    $num_protected++;
                if ($have_archive)
                    $num_have_archive++;
            }

            $group_size = $ipnet_group->get_channels($plugin_cookies)->size();
            if ($group_size == 0)
            {
//                hd_print("Warning: group $group_name (id=$group_id) is empty; skipped.");
            }
            else
            {
                $this->groups->put($ipnet_group);
                $num_groups++;
            }
        }

        hd_print("Ipnet: $num_groups groups and ".
            "$num_channels channels ($num_have_archive have archive, ".
            "$num_protected protected)");
    }

    public function get_tv_info(MediaURL $media_url, &$plugin_cookies)
    {
        $this->session->ensure_logged_in($plugin_cookies);

        return parent::get_tv_info($media_url, &$plugin_cookies);
    }

    ///////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////

    public function get_tv_stream_url($playback_url, &$plugin_cookies)
    {
        $this->session->check_logged_in();
		
        return $this->session->api_get_stream_url($playback_url);
    }

    public function get_tv_playback_url($channel_id, $archive_ts, $protect_code, &$plugin_cookies)
    {
		$this->session->try_login($plugin_cookies);
		
        $this->ensure_channels_loaded($plugin_cookies);

        if ($this->session->is_guest_login())
        {
            $url = sprintf(IPNET_GUEST_GET_URL_URL,
                Ipnet::$SERVER,
                $channel_id);
        }
        else
        {
            $url = sprintf(IPNET_GET_URL_URL,
                Ipnet::$SERVER,
                $this->session->get_sid_name(),
                $this->session->get_sid(),
                $channel_id);
        }

        if (isset($protect_code) && $protect_code !== '')
            $url .= "&protect_code=$protect_code";
        
		if (intval($archive_ts) > 0)
            $url .= "&gmt=$archive_ts&mode=http";
		else if ($this->session->get_use_multicast($plugin_cookies))
			$url .= "&mode=multicast";
		else
			$url .= "&mode=http";
		
		hd_print("get_tv_playback_url() URL = $url");
		
        return $url;
    }

    ///////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////

    public function get_day_epg_iterator($channel_id, $day_start_ts, &$plugin_cookies)
    {
        $this->ensure_channels_loaded($plugin_cookies);

        $day_str = gmstrftime('%d%m%y', $day_start_ts);
        if ($this->session->is_guest_login())
        {
            $url = sprintf(IPNET_GUEST_EPG_URL,
                Ipnet::$SERVER,
                $channel_id,
                $day_str);
        }
        else
        {
            $url = sprintf(IPNET_EPG_URL,
                Ipnet::$SERVER,
                $this->session->get_sid_name(),
                $this->session->get_sid(),
                $channel_id,
                $day_str);
        }

        try
        {
            $ipnet_epg = $this->session->api_call($url);
        }
        catch (Exception $e)
        {
            throw $this->session->dune_api_exception($e,
                'EPG request failed.',
                true);
        }

        $epg_items = array();

        foreach ($ipnet_epg->epg->item as $epg)
        {
            $progname = strval($epg->progname);

            $arr = explode("\n", $epg->progname, 2);
            $name = count($arr) >= 1 ? trim($arr[0]) : '';
            $description = count($arr) == 2 ? $arr[1] : '';

            $start = intval($epg->ut_start);
            $end = -1;

            $epg_items[] = new DefaultEpgItem(
                $name, $description, $start, $end);
        }

        return new EpgIterator($epg_items, $day_start_ts, $day_start_ts + 86400);
    }

    public function get_archive(MediaURL $media_url)
    {
        return $this->session->get_archive();
    }

    public function folder_entered(MediaURL $media_url, &$plugin_cookies)
    {
        if (!isset($media_url->screen_id) ||
            $media_url->screen_id === TvGroupListScreen::ID)
        {
            $this->session->logout();
        }

        $this->session->ensure_logged_in($plugin_cookies);
    }
}

///////////////////////////////////////////////////////////////////////////
?>
